/*------------------------------------------------------------------------
	File		    : TestScanner.p
	Purpose	    : Detect Proparse errors
	Syntax	    : Run TestScanner.p
	Description : Does a minimal parse of all files under a given directory
	            : in order to find problems with Proparse
	Author(s)   : Thomas Mercer-Hursh, Ph.D.
	Created	    : Tue Jun 16 13:48:00 CST 2015
	Notes	      : Requires prior process to create proparse.schema in chSchemaDirectory
  ----------------------------------------------------------------------*/
using Progress.Lang.*.
using com.cintegrity.Proparse.TestScanner.

block-level on error undo, throw.

/* ***************** Application Specific Definitions ***************** */
define variable chCompilableExtensions as character no-undo initial "p,w,cls".
define variable lgExcludeNoExtension as logical no-undo initial false.
define variable chExcludeDirectories as character no-undo initial "".
define variable chDataBaseList as character no-undo initial "c:\work\IS\DB\ISRel,c:\work\IS\DB\ISSort".
define variable chLogicalDataBaseList as character no-undo initial "ISRel,ISSort,DICTDB".
define variable chAliasList as character no-undo initial "".
define variable chDataBaseConnectParams as character no-undo initial "-1|-1".
define variable chSchemaDirectory as character no-undo initial "c:\work\IS\DB".
define variable chSchemaFileList as character no-undo initial "ISrel,ISsort,DICTDB".
/* Proparse schema file is proparse.schema in chSchemaDirectory */
/* Propath needed for compile */
define variable chPropath as character no-undo initial "c:\work\IS\Joe\src,c:\work\IS\ISrel\run,c:\work\IS\ISrel\src".
/* Source Variables */
define variable chSourceDirectoryList as character no-undo initial "c:\work\IS\Joe\src,c:\work\IS\ISrel\src".
define variable chXrefDirectoryList as character no-undo initial "c:\work\IS\Joe\xrf,c:\work\IS\ISrel\xrf".
/* Other variables */ 
define variable chPrimaryLog as character initial "c:\work\ABL2UML\ABL2DB\rpt\TestScanner.lg".
define variable chReportDestination as character no-undo initial "c:\work\IS\ISrel\rpt\TSCompileExceptions.txt".
define variable lgShowErrorDetails as logical no-undo initial false.
define variable inWhich as integer no-undo.
      
define variable obTestScanner as class TestScanner no-undo.

/* ***************************  Main Block  *************************** */
do on stop undo, retry:
	if retry
	then do:
		log-manager:write-message ("STOP condition encountered", "OtherERROR").
		undo, leave.
  	end.

	do on error undo, leave:
		session:error-stack-trace = true.
		log-manager:logfile-name = chPrimaryLog.
		log-manager:clear-log ().
		/* Control log detail and event types */
		log-manager:logging-level = 3.
		/* Uncomment for detailed trace during debugging */
		/* log-manager:log-entry-types = "DynObjects.Class,4GLTrace". */ 
		log-manager:write-message ("Start TestScanner").
    
    /* Detach Analysis, if in the context of ABL2DB and attach Databases for code being scanned */
    if connected( "Analysis" )
    then disconnect value( "Analysis" ) no-error.
    run AttachDatabases.
    propath = propath + "," + chPropath.
		/* ************************ Begin Actual Work ********************* */

    obTestScanner = new TestScanner( chSourceDirectoryList ).
    
    obTestScanner:chXrefDirectoryList = chXrefDirectoryList.
    obTestScanner:chIncludeExtensions = chCompilableExtensions.
    obTestScanner:lgExcludeNoExtension = lgExcludeNoExtension.
    obTestScanner:chExcludeDirectories = chExcludeDirectories.
    obTestScanner:lgExcludeEncrypted = false.
    obTestScanner:chReportDestination = chReportDestination.
    obTestScanner:lgShowErrorDetails = lgShowErrorDetails.
    
    obTestScanner:Initialize().
    obTestScanner:Process().

		/* Cleanup */

		if valid-object( obTestScanner ) then delete object obTestScanner.

		/* ************************ End Actual Work ********************* */
	
		catch eobAppError as Progress.Lang.AppError:
			log-manager:write-message ( eobAppError:ReturnValue ).	
			delete object eobAppError.
		end catch.
	
		catch eobSysError as Progress.Lang.SysError :
			log-manager:write-message(substitute("Unexpected Exception: &1", eobSysError:getMessage(1)), "SysERROR").
			if eobSysError:NumMessages > 1
			then do inWhich = 2 to eobSysError:NumMessages:
				log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SysERROR").
			end.
			do inWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
				log-manager:write-message(entry(inWhich,eobSysError:CallStack,"~n"),"CALLSTACK").
			end.
			delete object eobSysError no-error.
		end catch.
		
    catch eobOtherError as Progress.Lang.Error :
      log-manager:write-message(substitute("Other error: &2", eobOtherError:getMessage(1))).  
      if eobOtherError:NumMessages > 1
      then do inWhich = 2 to eobOtherError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobOtherError:getMessage(inWhich)), "OtherERROR").
      end.
      do inWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        log-manager:write-message(entry(inWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobOtherError no-error.
    end catch.

		finally:
			log-manager:write-message ("End TestScaner").
		end finally.

	end. /* do */
  
end.

procedure AttachDatabases:
  
  define variable minWhich as integer no-undo.
  define variable mchCurrentDB as character no-undo.
  define variable mchLogicalDB as character no-undo.
  
  do minWhich = 1 to num-entries( chDataBaseList ):
    mchCurrentDB = entry( minWhich, chDataBaseList ).
    mchLogicalDB = entry( minWhich, chLogicalDataBaseList ).
    connect value( mchCurrentDB ) value( entry( minWhich, chDataBaseConnectParams, "|" )).
    if not connected( mchLogicalDB )
    then do:
      log-manager:write-message(substitute("Database &1 (&2) did not connect", mchLogicalDB, mchCurrentDB )).
      undo, throw new Progress.Lang.AppError( "Failed to connect to needed database(s)").
    end.
    if chAliasList > "" and entry( minWhich, chAliasList ) <> ""
    then create alias value( entry( minWhich, chAliasList ) ) for database value( mchLogicalDB ).
  end.

  catch eobSysError as Progress.Lang.SysError :
    define variable inWhich as integer no-undo.
    log-manager:write-message(substitute("Driver: Unexpected Exception attaching database &1: &2", mchCurrentDB, eobSysError:getMessage(1)), "SYSERROR").
    if eobSysError:NumMessages > 1
    then do inWhich = 2 to eobSysError:NumMessages:
      log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SYSERROR").
    end.
  end catch.
  
end procedure.
