 /*------------------------------------------------------------------------
    File        : BuildDirectoryTree
    Purpose     : 
    Syntax      : 
    Description : Walks a directory tree and builds a list of all contents
    Author(s)   : Thomas Mercer-Hursh
    Created     : Thu Apr 10 16:26:03 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.FileSystem.PathUtilities.
using com.cintegrity.LoggingUtils.FlyWeightLogger.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildDirectoryTree: 
  define protected temp-table ttContents no-undo
     field chName as character
     field chExtension as character
     field chPackage as character
     field chSourceDirectory as character  
     field chFullPath as character
     field chSortName as character
     field chAttrs as character
     field lgIsDir as logical
     field lgIsPendingDir as logical
     field lgIsProcessed as logical
     index ixbyName as primary unique chFullPath
     index ixByDir lgIsDir
     index ixByProcessed lgIsProcessed
     index ixByPendingDir lgIsPendingDir
     index ixSort chSourceDirectory chSortName
     .
  define protected variable chSourceDirectoryList as character no-undo.
  define protected variable chSeparator as character no-undo.
  define protected variable chAntiSeparator as character no-undo.
  define protected variable inNumEntries as integer no-undo.

  define public property chReportDestination as character no-undo get. set.
  define public property chFileListDestination as character no-undo get. set.
  define public property chIncludeExtensions as character no-undo get. set.
  define public property chExcludeExtensions as character no-undo get. set.
  define public property lgExcludeNoExtension as logical no-undo get. set.
  define public property chExcludeDirectories as character no-undo get. set.
  define public property lgIncludeEmptyDirectories as logical no-undo initial false get. set.
  define public property lgExcludeEncrypted as logical no-undo initial true get. set.
  
  define protected variable obReport as class FlyWeightLogger no-undo. 
  define private variable obFileList as class FlyWeightLogger no-undo.
  define protected variable obPathUtilities as class PathUtilities no-undo.

  define stream stCurrentDir.
       
	constructor public BuildDirectoryTree ( 
	    ipchSourceDirectoryList as character
	    ):
		chSourceDirectoryList = ipchSourceDirectoryList.
		if num-entries(chSourceDirectoryList, "/") > num-entries(chSourceDirectoryList, "~\")
		then 
		  assign
		      chSeparator = "/"
		      chAntiSeparator = "~\"
		      .
		else 
		  assign
		      chSeparator = "~\"
		      chAntiSeparator = "/"
		      .
	end constructor.

  destructor public BuildDirectoryTree ( ):
    Reset().
    if valid-object( obPathUtilities ) then delete object obPathUtilities. 
    if valid-object(obReport)
    then do:
      obReport:CleanUp().
      delete object obReport.
    end.
    if valid-object(obFileList) then delete object obFileList.
  end destructor.
	
	method public void Initialize():
    obPathUtilities = new PathUtilities( ).
    obPathUtilities:Initialize().
    if chReportDestination > ""
    then do:
      obReport = new FlyWeightLogger().
      obReport:chLogDestination = chReportDestination. 
      obReport:Initialize( ).
    end.
    if chFileListDestination > ""
    then do:
      obFileList = new FlyWeightLogger().
      obFileList:chLogDestination = chFileListDestination. 
      obFileList:Initialize( ).
    end.
	end method.
	
	method public void Process():
	  define variable mchSourceDirectory as character no-undo.
	  define variable minWhich as integer no-undo.
	  
	  do minWhich = 1 to num-entries(chSourceDirectoryList,","):
	    mchSourceDirectory = entry(minWhich, chSourceDirectoryList, ",").
      file-info:file-name = mchSourceDirectory.
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*R*"
      then log-manager:write-message (substitute("BuildDirectoryTree ERROR: Source Path '&1' not found or not readable", mchSourceDirectory )).
      else ProcessTree ( mchSourceDirectory ).
  	end.
	end method.
	
	method public void Reset():
	  empty temp-table ttContents.
	end.

  method private void ProcessTree (
      ipchSourceDirectory as character
      ):
    define variable mlgNewAddition as logical no-undo.
    define variable minNumEntriesBefore as integer no-undo.
    define buffer bfContents for ttContents.
    
    LoadOneDirectory ( ipchSourceDirectory ).

    repeat:
      mlgNewAddition = false.    
        
      for each bfContents where bfContents.lgIsPendingDir = true:
        minNumEntriesBefore = inNumEntries.
        LoadOneDirectory ( bfContents.chFullPath ).
        if inNumEntries = minNumEntriesBefore
        then delete bfContents.
        else do:
          assign
              bfContents.lgIsPendingDir = false
              mlgNewAddition = true
              .
        end.
      end. /* for each */
      
      if mlgNewAddition = false then leave. 
      
    end. /* repeat */
    
  end method.
	
	method private void LoadOneDirectory (
	    ipchCurrentDir as character
	    ):
    define variable mchName as character no-undo.
    define variable mchFullPath as character no-undo.
    define variable mchRelativePath as character no-undo.
    define variable mchAttrs as character no-undo.
    define variable mlgIsDir as logical no-undo.
    define variable mchExtension as character no-undo initial "".
    define buffer bfContents for ttContents.
    
    if lookup(ipchCurrentDir,chExcludeDirectories) > 0 then return.
    	      
	  input stream stCurrentDir from os-dir( ipchCurrentDir ).
	  repeat:
	    import stream stCurrentDir mchName mchFullPath mchAttrs.

	    if mchName begins "." then next.
	    mchExtension = obPathUtilities:ExtensionFromName( mchName ).
	    mlgIsDir = index(mchAttrs, "D") > 0.
	    if not mlgIsDir
	    then do:
  	    if mchExtension > ""
  	    then do:
    	    if chIncludeExtensions > "" and lookup( mchExtension, chIncludeExtensions ) = 0 then next.
    	    if chExcludeExtensions > "" and lookup( mchExtension, chExcludeExtensions ) <> 0 then next.
  	    end.
  	    else if lgExcludeNoExtension then next.
  	    if      lgExcludeEncrypted
  	        and obPathUtilities:IsEncryptedSource ( mchFullPath )
  	    then next.
  	  end.
	    do transaction:
	      create bfContents.
	      assign
	          bfContents.chName = mchName
            bfContents.chExtension = mchExtension
            bfContents.chPackage = obPathUtilities:PackageFromFullPath( mchFullPath )
            bfContents.chSourceDirectory = obPathUtilities:SourceDirectoryFromFullPath( mchFullPath )
	          bfContents.chFullPath = mchFullPath
	          bfContents.chAttrs = mchAttrs
 	          bfContents.lgIsDir = mlgIsDir
	          bfContents.lgIsPendingDir = if bfContents.lgIsDir then true else ?
	          bfContents.lgIsProcessed = false
	          inNumEntries = inNumEntries + 1
	          .
	      if bfContents.lgIsDir
	      then do:
	        if bfContents.chPackage = ""
	        then bfContents.chSortName = bfContents.chName + chSeparator.
	        else bfContents.chSortName = bfContents.chPackage + chSeparator + bfContents.chName + chSeparator.
	      end.
	      else do:
          if bfContents.chPackage = ""
          then bfContents.chSortName = bfContents.chName.
          else bfContents.chSortName = bfContents.chPackage + chSeparator + bfContents.chName.
        end.	      
	    end.
	  end. /* repeat */  
	  input stream stCurrentDir close.    
	end method.
	
	method public void FileList():
	  define variable minBaseLevel as integer no-undo.
	  define variable mchLine as character no-undo.
	  define variable minNumFiles as integer no-undo.
	  
	  define buffer bfContents for ttContents.
	  
	  for each bfContents no-lock break by bfContents.chSourceDirectory by bfContents.chSortName:
	    if first-of( bfContents.chSourceDirectory )
	    then do:
        minBaseLevel = num-entries(bfContents.chSourceDirectory, chSeparator). 
        obFileList:WriteMessage ( substitute("Contents of &1", bfContents.chSourceDirectory) ).
        minNumFiles = 0.
	    end.
      mchLine = fill( " ", 2 * (num-entries(bfContents.chFullPath, chSeparator) - minBaseLevel)).
      mchLine = mchLine + bfContents.chName + " (" + bfContents.chSortName + ")".
      obFileList:WriteMessage ( mchLine ).
      minNumFiles = minNumFiles + 1.
      if last-of ( bfContents.chSourceDirectory )
      then do:
        mchLine = substitute("End of &1 with &2 total files", bfContents.chSourceDirectory, minNumFiles ).
        obFileList:WriteMessage ( mchLine ).
        obFileList:WriteMessage ( "-" ).
      end.
	  end. /* for each */
	end method.

end class.