/*------------------------------------------------------------------------
    File        : BuildDiskFiles
    Purpose     : Scan code base and build objects for all source code disk file units
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 15 14:51:48 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildDiskFiles inherits BuildDirectoryTree: 
  define public property chRunDirectoryList as character no-undo get. set.
  define public property chXrefDirectoryList as character no-undo get. set.
  define public property chListDirectoryList as character no-undo get. set.
  define public property chDebugDirectoryList as character no-undo get. set. 

  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.

  define stream stExceptions.

	constructor public BuildDiskFiles ( 
      ipchSourceDirectory as character
      ):  
    super ( ipchSourceDirectory ).
	end constructor.

	destructor public BuildDiskFiles ( ):
    input stream stExceptions close.   
    if valid-object(obReport) 
    then obReport:WriteMessage (substitute("Total files scanned = &1 of which &2 are compiled", inTotalFiles, inTotalCompiled )).
    log-manager:write-message ( substitute( "End BuildDiskFiles with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    define variable minWhich as integer no-undo.
    
    super:Initialize().

    log-manager:write-message ("Start BuildDiskFiles").
       
    do minWhich = 1 to num-entries( chSourceDirectoryList ):
    
      file-info:file-name = entry( minWhich, chRunDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("BuildDiskFiles ERROR: Run Path '&1' not found or not writeable", entry( minWhich, chRunDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "BuildDiskFiles: Unable to initialize." ).  
      end.      
      
      file-info:file-name = entry( minWhich, chXrefDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("BuildDiskFiles ERROR: XREF Path '&1' not found or not writeable", entry( minWhich, chXrefDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "BuildDiskFiles: Unable to initialize." ).  
      end.     
      
      file-info:file-name = entry( minWhich, chListDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("BuildDiskFiles ERROR: LIST Path '&1' not found or not writeable", entry( minWhich, chListDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "BuildDiskFiles: Unable to initialize." ).  
      end.     
      
      file-info:file-name = entry( minWhich, chDebugDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("BuildDiskFiles ERROR: DEBUG Path '&1' not found or not writeable", entry( minWhich, chDebugDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "BuildDiskFiles: Unable to initialize." ).  
      end.   
    
    end. /* do minWhich */
    
    obReport:WriteMessage ( substitute("Exception list for disk file build of &1", chSourceDirectoryList ) ).
  end method.
  
  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll( ).
    ResolveTriggers( ).
    
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not lgIsDir:
      ScanOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.

  method private void ScanOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchRunProgram as character no-undo.
    define variable mchXrefFile as character no-undo.
    define variable mchRCodeBaseDir as character no-undo.
    define variable mchListBaseDir as character no-undo.
    define variable mchXrefBaseDir as character no-undo.    
    define variable mchDebugBaseDir as character no-undo.
    define variable mlgIsCompiled as logical no-undo.
    define variable mlgIsClass as logical no-undo.
    define variable mchDiskFileID as character no-undo.
    define variable minLineCount as integer no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgIsEncrypted as logical no-undo.

    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
    define buffer bfCompileSubUnit for CompileSubUnit.
    
    minWhich = lookup( ipchSourceDirectory, chSourceDirectoryList ).
    if minWhich = 0
    then do:
      obReport:WriteMessage ( substitute("BuidldDiskFiles: Source direectory &1 not in Source Directory List for &2", ipchSourceDirectory, ipchFullPath ) ).
      inTotalExceptions = inTotalExceptions + 1.
      return.
    end.
    
    assign
        mchRCodeBaseDir = entry( minWhich, chRunDirectoryList )
        mchListBaseDir = entry( minWhich, chListDirectoryList )
        mchXrefBaseDir = entry( minWhich, chXrefDirectoryList )
        mchDebugBaseDir = entry( minWhich, chDebugDirectoryList )
        mchRunProgram = obPathUtilities:RunFile( mchRCodeBaseDir, ipchPackage, ipchName )
        mchXrefFile = obPathUtilities:XrefFile( mchXrefBaseDir, ipchPackage, ipchName )
        mlgIsEncrypted = obPathUtilities:IsEncryptedSource (ipchFullPath )
        inTotalFiles = inTotalFiles + 1
        .
    if not mlgIsEncrypted
    then do:    
      minLineCount = obPathUtilities:CountLines( ipchFullPath ).
      if minLineCount = ?
      then do:
        obReport:WriteMessage ( substitute("No line count returned for file &1", ipchFullPath ) ).
        inTotalExceptions = inTotalExceptions + 1.
      end.
    end.
    else do:
      obReport:WriteMessage ( substitute("File &1 is encrypted - limited processing possible", ipchFullPath ) ).
      inTotalExceptions = inTotalExceptions + 1.
    end.
    
    file-info:file-name = mchXrefFile.
    if file-info:full-pathname = ? 
    then mlgIsCompiled = no.
    else mlgIsCompiled = yes.    
    
    do for bfDiskFile:
      find bfDiskFile no-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if available bfDiskFile
      then do:
        obReport:WriteMessage ( substitute("DiskFile already exists for file &1", ipchFullPath ) ).
        inTotalExceptions = inTotalExceptions + 1.
        return.
      end.
      else do:
        create bfDiskFile.
        assign
            bfDiskFile.chID = guid
            bfDiskFile.chName = ipchName
            bfDiskFile.chPackage = ipchPackage
            bfDiskFile.chBaseDirectory = ipchSourceDirectory
            bfDiskFile.lgCompilable = mlgIsCompiled
            bfDiskFile.inLines = minLineCount
            bfDiskFile.lgIsEncrypted = mlgIsEncrypted
            mchDiskFileID = bfDiskFile.chID
            .
      end.
    end.  
   
    if mlgIsCompiled
    then do:
      inTotalCompiled = inTotalCompiled + 1.
      if ipchExtension = "cls"
      then mlgIsClass = true.
      else mlgIsClass = false.
      do for bfCompileUnit:
        create bfCompileUnit.
        assign
            bfCompileUnit.chID = guid
            bfCompileUnit.chRCodeBaseDir = mchRCodeBaseDir
            bfCompileUnit.chListBaseDir = mchListBaseDir
            bfCompileUnit.chXrefBaseDir = mchXrefBaseDir
            bfCompileUnit.chDebugBaseDir = mchDebugBaseDir
            bfCompileUnit.lgIsClass = mlgIsClass
            bfCompileUnit.chDiskFileID = mchDiskFileID 
            .          
      end.
      if mlgIsClass = false
      then do for bfCompileSubUnit:      /* Create ProgramRoot for Procedures */
        create bfCompileSubUnit.
        assign
            bfCompileSubUnit.chID = guid
            bfCompileSubUnit.chName = ""
            bfCompileSubUnit.chType = "ProgramRoot"
            bfCompileSubUnit.chCompileUnitID = chCompileUnitID
            .
      end.
    end.
  end method.
  
  method private void ResolveTriggers( ):
    define buffer bfDB_TableTrigger for DB_TableTrigger.
    define buffer bfDiskFile for DiskFile.
    define buffer bfDB_Table for DB_Table.
    define variable mchSourceDirectory as character no-undo.
    define variable mchPackage as character no-undo.
    define variable mchName as character no-undo.
    
    for each bfDB_TableTrigger exclusive-lock:
      if bfDB_TableTrigger.chProcedureName > ""
      then do for bfDiskFile:
        mchSourceDirectory = obPathUtilities:SourceDirectoryFromRelativePath( bfDB_TableTrigger.chProcedureName ).    
        mchPackage = obPathUtilities:PackageFromRelativePath( bfDB_TableTrigger.chProcedureName ). 
        mchName = obPathUtilities:NameFromPath( bfDB_TableTrigger.chProcedureName ). 
        find bfDiskFile no-lock
            where bfDiskFile.chBaseDirectory = mchSourceDirectory
            and   bfDiskFile.chPackage = mchPackage
            and   bfDiskFile.chName = mchName
            no-error.
        if available( bfDiskFile )
        then do:
          bfDB_TableTrigger.chProcedureID = bfDiskFile.chID.
        end.
        else do for bfDB_Table:
          find bfDB_Table no-lock where bfDB_Table.chID = bfDB_TableTrigger.chTableID.
          obReport:WriteMessage ( substitute("No DiskFile found for &1 trigger on &2 with procedure &3|&4|&5 (&6)", bfDB_TableTrigger.chEvent, bfDB_Table.chTableName, mchSourceDirectory, mchPackage, mchName, bfDB_TableTrigger.chProcedureName ) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
      end.
    end. /* for bfTableTrigger */
  end method.
  
end class.