/*------------------------------------------------------------------------
    File        : BuildMenu
    Purpose     : 
    Syntax      : 
    Title : Load menu and create functional units
    Author(s)   : Thomas Mercer-Hursh
    Created     : Fri May 09 11:43:25 CDT 2014
    Notes       : Since menu structures vary widely across applications
                  this class is designed to load the menu from a 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.FileSystem.PathUtilities.
using com.cintegrity.LoggingUtils.FlyWeightLogger.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildMenu: 
  define public property chSourceDirectoryList as character no-undo get. set.
  define public property chMenuFile as character no-undo get. set.
  define public property chMenuItemFile as character no-undo get. set.
  define public property chFunctionalUnitFile as character no-undo get. set.
  define public property chReportDestination as character no-undo get. set.

  define public property inTotalMenus as integer no-undo get. private set.
  define public property inTotalMenuItems as integer no-undo get. private set.
  define public property inTotalMenuFunctionalUnits as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. set.
  
  define private variable obReport as class FlyWeightLogger no-undo. 
  define private variable obPathUtilities as class PathUtilities no-undo.
  
  define stream stMenu.
  define stream stMenuItem.
  define stream stFunctUnit.
		
	constructor public BuildMenu (  ):
		
	end constructor.

	destructor public BuildMenu ( ):
    input stream stMenu close.
    input stream stMenuItem close.
    input stream stFunctUnit close.
    if valid-object(obReport) 
    then obReport:WriteMessage ( substitute("BuildMenu concluded with &1 menus, &2 menu items, and &3 functional units", inTotalMenus, inTotalMenuItems, inTotalMenuFunctionalUnits )).
    delete object obPathUtilities.
    delete object obReport.
    log-manager:write-message ( substitute( "End of BuildMenu with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public void Initialize ( ):
    log-manager:write-message ( "Start of BuildMenu").
    file-info:file-name = chMenuFile.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      log-manager:write-message (substitute("BuildMenu ERROR: MenuFile '&1' not found or not writeable", chMenuFile )).
      undo, throw new Progress.Lang.AppError( "BuildMenu: Unable to initialize." ).  
    end.  

    file-info:file-name = chMenuItemFile.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      log-manager:write-message (substitute("BuildMenu ERROR: MenuItemFile '&1' not found or not writeable", chMenuItemFile )).
      undo, throw new Progress.Lang.AppError( "BuildMenu: Unable to initialize." ).  
    end.  

    file-info:file-name = chFunctionalUnitFile.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      log-manager:write-message (substitute("BuildMenu ERROR: chFunctionalUnitFile '&1' not found or not writeable", chFunctionalUnitFile )).
      undo, throw new Progress.Lang.AppError( "BuildMenu: Unable to initialize." ).  
    end.  
    
    input stream stMenu from value(chMenuFile).
    input stream stMenuItem from value(chMenuItemFile).
    input stream stFunctUnit from value(chFunctionalUnitFile).
    
    obPathUtilities = new PathUtilities( ).
    obPathUtilities:Initialize().
    obReport = new FlyWeightLogger().
    obReport:chLogDestination = chReportDestination. 
    obReport:Initialize( ).

    obReport:WriteMessage ( substitute("Exception list for menu and functional unit build from &1 and &2", chMenuFile, chFunctionalUnitFile ) ).
  end method.
  
  method public void Process ( ):
    ProcessMenus().
    ProcessFunctionalUnits().
    ProcessMenuItems().
  end method.
  
  method private void ProcessMenus ( ):
    define variable mchMenuName as character no-undo.
    define variable mchMenuReference as character no-undo.
    define variable mchmenuTitle as character no-undo.
    define variable minLine as integer no-undo.
    
    define buffer bfAppMenu for AppMenu.
    
    repeat for bfAppMenu on error undo, throw:
      import stream stMenu mchMenuName mchMenuTitle.

      create bfAppMenu.
      assign 
          minLine = minLine + 1
          inTotalMenus = inTotalMenus + 1
          bfAppMenu.chID = guid
          bfAppMenu.chMenuName = mchMenuName
          bfAppMenu.chMenuTitle = mchMenuTitle
          .
      
      catch eobSysError as Progress.Lang.SysError :
        obReport:WriteMessage ( substitute("BuildMenu: Error importing menu entry with name = &1 and title = &2 at line &3", mchMenuName, mchMenuTitle, minLine ) ).
        obReport:WriteMessage ( eobSysError:getMessage(1) ).
        inTotalExceptions = inTotalExceptions + 1.  
        delete object eobSysError.
      end catch.
    end.
  end method.

  method private void ProcessFunctionalUnits ( ):
    define variable mchFunctionalUnitName as character no-undo.
    define variable mchFunctionalUnitTitle as character no-undo.
    define variable mchCompileUnitPath as character no-undo.
    define variable minLine as integer no-undo.
    define variable mchBaseDirectory as character no-undo.
    define variable mchPackage as character no-undo.
    define variable mchName as character no-undo.
    
    define buffer bfFunctionalUnit for FunctionalUnit.
    define buffer bfCompileUnit for CompileUnit.
    define buffer bfDiskFile for DiskFile.
    
    repeat for bfFunctionalUnit on error undo, throw:
      import stream stFunctUnit mchFunctionalUnitName mchFunctionalUnitTitle mchCompileUnitPath.
      
      create bfFunctionalUnit.
      assign
          minLine = minLine + 1
          inTotalMenuFunctionalUnits = inTotalMenuFunctionalUnits + 1
          bfFunctionalUnit.chID = guid
          bfFunctionalUnit.chName = mchFunctionalUnitName
          bfFunctionalUnit.chDescription = mchFunctionalUnitTitle
          mchBaseDirectory = obPathUtilities:SourceDirectoryFromRelativePath ( mchCompileUnitPath )
          mchPackage = obPathUtilities:PackageFromRelativePath ( mchCompileUnitPath )
          mchName = obPathUtilities:NameFromPath ( mchCompileUnitPath )
          .
      do for bfDiskFile:
        find bfDiskFile no-lock
            where bfDiskFile.chBaseDirectory = mchBaseDirectory
            and   bfDiskFile.chPackage = mchPackage
            and   bfDiskFile.chName = mchName
            no-error.
        if not available ( bfDiskFile )
        then do:
          obReport:WriteMessage ( substitute("BuildMenu: DiskFile not found for FunctionalUnit = &1 with CompileUnitPath = &2 at line &3", mchFunctionalUnitName, mchCompileUnitPath, minLine ) ).
          inTotalExceptions = inTotalExceptions + 1.           
        end.
        else do for bfCompileUnit:
          find bfCompileUnit exclusive-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
          if not available( bfCompileUnit )
          then do:
            obReport:WriteMessage ( substitute("BuildMenu: CompileUnit for &1 is missing", mchCompileUnitPath ) ). 
            inTotalExceptions = inTotalExceptions + 1.
          end.
          else bfFunctionalUnit.chMainCompileUnitID = bfCompileUnit.chID.
        end.
      end.
      
      catch eobSysError as Progress.Lang.SysError :
        obReport:WriteMessage ( substitute("BuildMenu: Error importing functional unit entry with name = &1 and title = &2 at line &3", mchFunctionalUnitName, mchFunctionalUnitTitle, minLine ) ).
        obReport:WriteMessage ( eobSysError:getMessage(1) ).  
        inTotalExceptions = inTotalExceptions + 1.
        delete object eobSysError.
      end catch.      
    end.
  end method.
  
  method private void ProcessMenuItems ( ):
    define variable mchMenuName as character no-undo.
    define variable mchSeq as character no-undo.
    define variable minSeq as integer no-undo.
    define variable mchIsMenu as character no-undo.
    define variable mlgIsMenu as logical no-undo.
    define variable mchMenuID as character no-undo.
    define variable mchLinkName as character no-undo.
    define variable mchSubMenuID as character no-undo.
    define variable mchFunctionalUnitID as character no-undo.
    define variable minLine as integer no-undo.
    
    define buffer bfAppMenuItem for AppMenuItem.
    define buffer bfAppMenu for AppMenu.
    define buffer bfAppSubMenu for AppMenu.
    define buffer bfFunctionalUnit for FunctionalUnit.
    define buffer bfAppMenuItemSubMenu for AppMenuItemSubMenu.
    define buffer bfAppMenuItemFunctionalUnit for AppMenuItemFunctionalUnit.
    
    repeat for bfAppMenuItem on error undo, throw:
      import stream stMenuItem mchMenuName mchSeq mchIsMenu mchLinkName.

      assign 
          minLine = minLine + 1
          inTotalMenuItems = inTotalMenuItems + 1
          minSeq = integer(mchSeq)
          mlgIsMenu = false
          mlgIsMenu = true when mchIsMenu = "yes"
          mchMenuID = ""
          mchSubMenuID = ""
          mchFunctionalUnitID = ""
          .
          
      do for bfAppMenu:
        find bfAppMenu no-lock where bfAppMenu.chMenuName = mchMenuName no-error.
        if not available ( bfAppMenu )
        then do:
          obReport:WriteMessage ( substitute("BuildMenu: AppMenu not found for Menu = &1 #&2 at line &3", mchMenuName, mchSeq, minLine ) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
        else mchMenuID = bfAppMenu.chID.
      end.            
      
      create bfAppMenuItem.
      assign
          bfAppMenuItem.chID = guid
          bfAppMenuItem.chMenuID = mchMenuID
          bfAppMenuItem.inSeq = minSeq
          bfAppMenuItem.lgIsMenu = lgIsMenu
          .
      
      if mlgIsMenu
      then do for bfAppSubMenu:
        find bfAppSubMenu no-lock where bfAppSubMenu.chMenuName = mchLinkName no-error.
        if not available ( bfAppSubMenu )
        then do:
          obReport:WriteMessage ( substitute("BuildMenu: AppMenu not found for MenuItem = &1 #&2 with SubMenuName = &3 at line &4", mchMenuName, mchSeq, mchLinkName, minLine ) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
        else do for bfAppMenuItemSubMenu:
          create bfAppMenuItemSubMenu.
          assign
              bfAppMenuItemSubMenu.chID = guid
              bfAppMenuItemSubMenu.chMenuItemID = bfAppMenuItem.chID
              bfAppMenuItemSubMenu.chSubMenuId = bfAppSubMenu.chID
              .
        end.
      end.
      else do for bfFunctionalUnit:
        find bfFunctionalUnit no-lock where bfFunctionalUnit.chName = mchLinkName no-error.
        if not available ( bfFunctionalUnit )
        then do:
          obReport:WriteMessage ( substitute("BuildMenu: FunctionalUnit not found for MenuItem = &1 #&2 with FunctionalUnitName = &3 at line &4", mchMenuName, mchSeq, mchLinkName, minLine ) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
        else do for bfAppMenuItemFunctionalUnit:
          create bfAppMenuItemFunctionalUnit.
          assign
              bfAppMenuItemFunctionalUnit.chID = guid
              bfAppMenuItemFunctionalUnit.chMenuItemID = bfAppMenuItem.chID
              bfAppMenuItemFunctionalUnit.chFunctionalUnitId = bfFunctionalUnit.chID
              .
        end.
      end. 

      catch eobSysError as Progress.Lang.SysError :
        obReport:WriteMessage ( substitute("BuildMenu: Error importing menu entry with name = &1 #2 => &3 at line &4", mchMenuName, mchSeq, mchLinkName, minLine ) ).
        obReport:WriteMessage ( eobSysError:getMessage(1)  ).  
        inTotalExceptions = inTotalExceptions + 1.
        delete object eobSysError.
      end catch.
    end. /* AppMenuItem */      
  end method.
  
end class.