/*------------------------------------------------------------------------
    File        : PathUtilities
    Purpose     : 
    Syntax      : 
    Description : Collection of Utility Methods relating to PROPATH, Package, and File Name
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sat May 03 10:40:13 CDT 2014
    Notes       : Vocabulary
                  SourceDirectory = the portion of the propath in which the package is found (includes Drive on Windows)
                  Package = the path, if any, from the SourceDirectory to the file
                  Name = the filename within the package, including the extension
                  RelativePath = Path and name from PROPATH base, i.e., relative to SourceDirectory
                  Extension = the extension, if any, part of the name
                  FullPath = SourceDirectory, Package, and Name composed into an absolute file reference (includes Drive on Windows)
                  Separator = the file system appropriate connector, i.e., \ or /
                  
                  Note that Windows drive designators are required in the PROPATH
                  Note that it is assumed that any path passed in contains the drive letters when in Windows
 ----------------------------------------------------------------------*/

using Progress.Lang.*.

block-level on error undo, throw.

class com.cintegrity.FileSystem.PathUtilities: 
  define private property chPropath as character no-undo get. private set.
  define private property chSeparator as character no-undo get. private set.
  define private property chAntiSeparator as character no-undo get. set.
  
  define private temp-table ttPropath no-undo
    field inOrder as integer
    field chPath as character
    index ixByOrder as primary unique inOrder
    . 
		
	constructor public PathUtilities ( ):
    chPropath = propath.
	end constructor.

	destructor public PathUtilities ( ):

	end destructor.
	
	method public void Initialize ():
    define variable minWhich as integer no-undo.
    define variable mchEntry as character no-undo.
    define buffer bfttPropath for ttPropath.
    
    if num-entries(chPropath, "/") > num-entries(chPropath, "~\")
    then do:
      chSeparator = "/".
      chAntiSeparator = "~\".
    end.
    else do:
      chSeparator = "~\".
      chAntiSeparator = "/".
    end.
    
    do for bfttPropath minWhich = 1 to num-entries(chProPath, ","):
      mchEntry = entry(minWhich, chPropath, ",").
      file-info:file-name = mchEntry.
      mchEntry = file-info:full-pathname.
      if mchEntry = ?
      then do:
        log-manager:write-message (substitute("PathUtilities: Propath entry &1 not found in filesystem", entry(minWhich, chPropath, ",") )).
        undo, throw new Progress.Lang.AppError( "Aborting." ).         
      end.
      else do:
        create bfttPropath.
        assign
            bfttPropath.inOrder = minWhich
            bfttPropath.chPath = mchEntry
            .        
      end.
    end.	  
	end method.
	                                        /* Construct paths from components */
	method public character FullPathFromComponents (
	    ipchSourceDirectory as character,
	    ipchPackage as character,
	    ipchName as character
	    ):
	  define variable mchFullPath as character no-undo.
	  
    if ipchPackage > ""
    then mchFullPath = ipchSourceDirectory + chSeparator + ipchPackage + chSeparator + ipchName.
    else mchFullPath = ipchSourceDirectory + chSeparator + ipchName.	   
    return mchFullPath.   
  end method.
  
  method public character FullPathFromComponents (
      ipchDirectory as character,
      ipchName as character
      ):
    return ipchDirectory + chSeparator + ipchName.     
  end method.
  
  method public character RelativePathFromComponents (
      ipchPackage as character,
      ipchName as character
      ):
    define variable mchRelativeName as character no-undo.
      
    if ipchPackage > ""
    then mchRelativeName = ipchPackage + chSeparator + ipchName.
    else mchRelativeName = ipchName.
    return mchRelativeName.
  end method.
	                                                 /* Obtain SourceDirectory */
  method public character SourceDirectoryFromFullPath (
      ipchFullPath as character
      ):
    define variable mchSourceDirectory as character no-undo initial "NOT FOUND".
    define buffer bfttPropath for ttPropath.
    
    for each bfttPropath by bfttPropath.inOrder:
      if substring(ipchFullPath, 1, length(bfttPropath.chPath)) = bfttPropath.chPath
      then do:
        mchSourceDirectory = bfttPropath.chPath.
        leave.
      end.
    end.
    if mchSourceDirectory = ""
    then log-manager:write-message ( substitute("PathUtilities - SourceDirectoryFromFullPath: Source Directory not found on Propath for path &1", ipchFullPath)).
    return mchSourceDirectory.
  end method.
  
  method public character SourceDirectoryFromRelativePath (
      ipchRelativePath as character
      ):
    define variable chFullPath as character no-undo.
    define variable chSourceDirectory as character no-undo.
    
    chFullPath = search(ipchRelativePath).
    if chFullPath ne ?
    then chFullPath = replace(chFullPath, chSeparator + ipchRelativePath, "").
    return chFullPath. 
  end method.
                                                           /* Obtain Package */
  method public character PackageFromFullPath (
      ipchFullPath as character
      ):
    define variable mchRelativePath as character no-undo initial "".
    define variable mchPackage as character no-undo initial "NOT FOUND".
    define buffer bfttPropath for ttPropath.

    for each bfttPropath by bfttPropath.inOrder:
      if substring(ipchFullPath, 1, length(bfttPropath.chPath)) = bfttPropath.chPath
      then do:
        mchRelativePath = substring(ipchFullPath, length(bfttPropath.chPath) + 2). 
        leave.
      end.
    end.    
    if length(mchRelativePath) > 0
    then mchPackage = PackageFromRelativePath( mchRelativePath ).
    else log-manager:write-message ( substitute("PathUtilities - PackageFromFullPath: Package not found on Propath for path &1", ipchFullPath)).
    return mchPackage.
  end method.

  method public character PackageFromRelativePath (
      ipchRelativePath as character
      ):
    define variable mchPackage as character no-undo.
    
    if r-index(ipchRelativePath, chSeparator) > 0
    then mchPackage = substring(ipchRelativePath, 1, r-index(ipchRelativePath, chSeparator) - 1).
    else mchPackage = "".
    return mchPackage.
  end method.
                                                              /* Obtain Name */
  method public character NameFromPath (
      ipchPath as character               /* Works for Full or Relative Path */
      ):
    define variable mchName as character no-undo.
    define buffer bfttPropath for ttPropath.
    
    mchName = entry(num-entries(ipchPath, chSeparator), ipchPath, chSeparator ).
    return mchName.
  end method.
                                                         /* Obtain Extension */
  method public character ExtensionFromName (
      ipchName as character
      ):
    define variable mchExtension as character no-undo.    
       
    if num-entries(ipchName, ".") > 1 
    then mchExtension = entry(num-entries(ipchName, "."), ipchName, "." ).
    return mchExtension.    
  end.
                                                    /* Compile related names */
  method public character RunFile (
      ipchRunDestination as character,
      ipchPackage as character,
      ipchName as character
      ):
    define variable mchRunFile as character no-undo.
    
    if ipchPackage > ""    
    then mchRunFile = ipchRunDestination + chSeparator + ipchPackage + chSeparator + ipchName.
    else mchRunFile = ipchRunDestination + chSeparator + ipchName.
    if mchRunFile matches "*~~.cls" then mchRunFile = substring(mchRunFile, 1, length(mchRunFile) - 3 ) + "r".  
    else if mchRunFile matches "*~~.p" then mchRunFile = substring(mchRunFile, 1, length(mchRunFile) - 1 ) + "r".
    else mchRunFile = mchRunFile + ".r".
    return mchRunFile.          
  end method.

  method public character ListFile (
      ipchListDestination as character,
      ipchPackage as character,
      ipchName as character
      ):
    define variable mchListFile as character no-undo.
    
    if ipchPackage > ""    
    then mchListFile = ipchListDestination + chSeparator + ipchPackage + chSeparator + ipchName.
    else mchListFile = ipchListDestination + chSeparator + ipchName.
    mchListFile = mchListFile + ".lst".
    return mchListFile.          
  end method.

  method public character XrefFile (
      ipchXrefDestination as character,
      ipchPackage as character,
      ipchName as character
      ):
    define variable mchXrefFileName as character no-undo.
    define variable mchXrefFile as character no-undo.
    
    if ExtensionFromName( ipchName ) > ""
    then mchXrefFileName = substring(ipchName, 1, r-index(ipchName, "." + ExtensionFromName( ipchName )) - 1) + ".xref.xml".
    else mchXrefFileName = ipchName + ".xref.xml".
    
    if ipchPackage > ""
    then mchXrefFile = ipchXrefDestination + chSeparator + ipchPackage + chSeparator + mchXrefFileName.
    else mchXrefFile = ipchXrefDestination + chSeparator + mchXrefFileName.
    
    return mchXrefFile.        
  end method.
  
  method public character DebugFile (
      ipchDebugDestination as character,
      ipchPackage as character,
      ipchName as character
      ):
    define variable mchDebugFile as character no-undo.
    
    if ipchPackage > ""    
    then mchDebugFile = ipchDebugDestination + chSeparator + ipchPackage + chSeparator + ipchName.
    else mchDebugFile = ipchDebugDestination + chSeparator + ipchName.
    mchDebugFile = mchDebugFile + ".debug".
    return mchDebugFile.          
  end method.
                                                         /* Create Directory */
  method public void MakeDirectory(
      ipchDirectory as character
      ):
    file-info:file-name = ipchDirectory.
    if file-info:full-pathname = ?
    then do:
      MakeDirectory( substring(ipchDirectory,1, r-index(ipchDirectory, chSeparator ) - 1)).
      os-create-dir value( ipchDirectory ).
      if os-error <> 0
      then do:
        log-manager:write-message (substitute("PathUtilities: OS-ERROR &1 creating directory '&2'", string(os-error), ipchDirectory )).
        undo, throw new Progress.Lang.AppError( "Aborting." ).  
      end.
    end.         
  end method.
                                                      /* Count lines in file */
  method public integer CountLines (
      ipchSourceFile as character
      ):
    define variable mlcFile as longchar no-undo.
    define variable minLineCount as integer no-undo.
    define variable inWhich as integer no-undo.
    
    file-info:file-name = ipchSourceFile.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:    
      log-manager:write-message ( substitute( "CountLines of PathUtilities: &1 is not a file or is not readable", ipchSourceFile)).
      return ?.
    end.
    else do:
      copy-lob from file ipchSourceFile to mlcFile.
      minLineCount = num-entries( mlcFile, "~n" ).
      return minLineCount.
    end.
    catch eobSysError as Progress.Lang.SysError :
      log-manager:write-message(substitute("PathUtilities: Countlines: Problem with copy-lob of file &1",ipchSourceFile)).
      log-manager:write-message(substitute("Unexpected Exception: &1", eobSysError:getMessage(1)), "SYSERROR").
      if eobSysError:NumMessages > 1
      then do inWhich = 2 to eobSysError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SYSERROR").
      end.
      delete object eobSysError no-error.
    end catch.
  end method.
                       /* Match "Mangled" name as shown in List block output */
  method public character MangledNameFromPath (
      ipchFullPath as character
      ):
    define variable mchMangledName as character no-undo.
    define variable minLen as integer no-undo.
    
    minLen = length(ipchFullPath).
    mchMangledName = "..." + substring( ipchFullPath, minLen - 16, 17 ).
    return mchMangledName.
  end method.
  
  method public logical IsEncryptedSource (
      ipchFullPath as character
      ):
    input from value(ipchFullPath) binary.
    readkey.
    input close.

    if     lastkey = 19 
        or lastkey = 17
    then return true.
    else return false.     
  end method.

  method private character CorrectSeparator (
       ipchPath as character
       ):
    if num-entries(ipchPath, chAntiSeparator ) > 1
    then return replace( ipchPath, chAntiSeparator, chSeparator ).
    else return ipchPath.
  end method.
      
end class.