/*------------------------------------------------------------------------
   File        : TokenLister
   Purpose     : 
   Syntax      : 
   Description : Export the tokens parse by Proparse.
   Author(s)   : Thomas Mercer-Hursh
   Created     : Mon Mar 05 16:01:05 PST 2012
   Usage       : Set properties needed, Initialize(), DoList()
               : Assumes proparse.schema has been previously generated in chSchemaDirector
   Notes       : Set up variables as follows
               : chSchemaDirectory - Directory containing proparse.schema
               : chSchemaDumpFile - name of file containing schema, (default proparse.schema)
               : chDataBaseList - comma delimited list of logical DB names (not necessarily all connected)
               : chAliasList - comma delimited list of aliases
               : chFileToParse - the name of the file to process
               : chOutputFile -  where to put the listing
               : inTabSize - size of indent (default 4)
               : lgShowColumn - show column number in list (default false)
               : lgShowProparseDirectives - show Proparse directives (default false)
               : lgShowSourceFileName - show source file for each token (default true)
               : lgShowSourceLineNo - show line number (default true)
               : lgShowSecondaryKeyword - show secondary keyword, like buffer for define (default false)
               : lgShowStatementHeadFlag - show flag for statement head (default false)
               : lgShowTableType - show table type, i.e., db, tt, or wt (default true)
   Derived roughly from proparse/api/tokenlister.p 2001 by John Green, Joanju Limited
 ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.core.TreeUtils from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.
using org.prorefactor.treeparser.Symbol from assembly.

routine-level on error undo, throw.

class com.cintegrity.Proparse.TokenLister: 
  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character no-undo get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chFileToParse as character no-undo get. set.
  define public property chOutputFile as character no-undo get. set.
  define public property inTabSize as integer no-undo initial 4 get. set.
  define public property lgShowColumn as logical no-undo initial false get. set.
  define public property lgShowProparseDirectives as logical no-undo initial false get. set.
  define public property lgShowSourceFileName as logical no-undo initial true get. set.
  define public property lgShowSourceLineNo as logical no-undo initial true get. set.
  define public property lgShowSecondaryKeyword as logical no-undo initial false get. set.
  define public property lgShowStatementHeadFlag as logical no-undo initial false get. set.
  define public property lgShowTableType as logical no-undo initial true get. set.
  define public property lgShowSymbols as logical no-undo initial true get. set.
  
  define private variable inCurrentIndent as integer no-undo initial 0.
  define private variable inCurrentLine as integer no-undo initial 0.
  define private variable inFirstNode as integer no-undo.
  define private variable inGrandChild as integer no-undo.

  define private variable obSetEnvironment as class SetEnvironment no-undo.
  define private variable obSetSchemaFromFile as class SetSchemaFromFile no-undo.
  
  define private stream stOutput.
  
  constructor public TokenLister ( ):
    assign chOutputFile = session:temp-directory + "/TokenLister.txt":U.
  end constructor.
  
  destructor public TokenLister ( ):

  end destructor.
  
  method public void Initialize():
    obSetEnvironment = new SetEnvironment().
    obSetEnvironment:Initialize().
    obSetSchemaFromFile = new SetSchemaFromFile().
    obSetSchemaFromFile:chSchemaDirectory = chSchemaDirectory.
    obSetSchemaFromFile:chSchemaDumpFile = chSchemaDumpFile.
    obSetSchemaFromFile:chLogicalDataBaseList = chLogicalDataBaseList.  
    obSetSchemaFromFile:chAliasList = chAliasList.     
    obSetSchemaFromFile:Initialize().
  end method.
  
  method public void DoList():
    define variable mchFullPath as character no-undo.
    define variable mobJavaFile as class java.io.File no-undo.
    define variable mobNode as class JPNode no-undo.
    define variable obParseUnit as class ParseUnit no-undo.
    define variable minWhich as integer no-undo.
    
    if lgShowProparseDirectives 
    then obSetEnvironment:SetShowDirectives( true ).
    else obSetEnvironment:SetShowDirectives( false ).

    output stream stOutput to value(chOutputFile).
     
    do on error undo, leave:
      file-info:file-name = chFileToParse.
      mchFullPath = file-info:full-pathname.
      if mchFullPath = ?
      then undo, throw new AppError ( substitute( "File to Parse `&1` not found", chFileToParse ) ).
      
      put stream stOutput unformatted substitute( "Parse of &1 at &2", mchFullPath, now ) skip.
      
      mobJavaFile = new java.io.File(mchFullPath).
      obParseUnit = new ParseUnit(mobJavaFile).
      obParseUnit:treeParser01().
      mobNode = obParseUnit:getTopNode().   
      TreeWalker( mobNode ).

      catch eobAppError as Progress.Lang.AppError:
        put stream stOutput unformatted substitute("AppError: &1", eobAppError:ReturnValue ) skip.  
        delete object eobAppError.
      end catch.
      
      catch eobOtherError as Progress.Lang.Error :
        put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
        if eobOtherError:NumMessages > 1
        then do minWhich = 2 to eobOtherError:NumMessages:
          put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
        end.
        do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
          put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
        end.
        delete object eobOtherError no-error.
      end catch.
    end. /* do */

    output stream stOutput close.

  end method. /* DoList */  
  
  method private void TreeWalker(
      ipobNode as class JPNode
      ): 
    if (ipobNode eq ?) then return.
    inCurrentLine = ipobNode:getLine().
    PrintNode( ipobNode ).
    assign inCurrentIndent = inCurrentIndent + inTabSize.
    TreeWalker( ipobNode:firstChild() ).
    assign inCurrentIndent = inCurrentIndent - inTabSize.
    TreeWalker( ipobNode:nextSibling() ).        

  end method. /* TreeWalker */
  
  method private void PrintNode(
      ipobNode as class JPNode
      ):
    define variable mobSymbol as class Symbol no-undo.
    
    if NodeTypes:getTypeName(ipobNode:getType()) = "ID"
    then mobSymbol = ipobNode:getSymbol().
        
    put stream stOutput unformatted
      skip
      fill(" ",inCurrentIndent)
      NodeTypes:getTypeName(ipobNode:getType()) 
      "    "
      ipobNode:getText()
      .
    if lgShowProparseDirectives then put stream stOutput unformatted  "    " ipobNode:attrGetS("proparsedirective":U).
    if lgShowSourceLineNo or lgShowColumn then put stream stOutput unformatted  "    " string(ipobNode:getLine()).
    if lgShowColumn then put stream stOutput unformatted  ":" string(ipobNode:getLine()).
    if lgShowSourceFileName then put stream stOutput unformatted  "    " ipobNode:getFilename().
    if lgShowTableType then put stream stOutput unformatted  "    " ipobNode:attrGetS("storetype":U).
    if lgShowStatementHeadFlag and ipobNode:isStateHead() then put stream stOutput unformatted  "    statehead".
    if lgShowSecondaryKeyword then put stream stOutput unformatted  "    " ipobNode:getState2().
    if lgShowSymbols and NodeTypes:getTypeName(ipobNode:getType()) = "ID" and valid-object(mobSymbol)
    then put stream stOutput unformatted  "    Reads: " mobSymbol:getNumReads() " Writes: " mobSymbol:getNumWrites().
    put stream stOutput unformatted skip.
  end method. /* PrintNode */

end class.
